/* -LICENSE-START-
** Copyright (c) 2013 Blackmagic Design
**
** Permission is hereby granted, free of charge, to any person or organization
** obtaining a copy of the software and accompanying documentation covered by
** this license (the "Software") to use, reproduce, display, distribute,
** execute, and transmit the Software, and to prepare derivative works of the
** Software, and to permit third-parties to whom the Software is furnished to
** do so, all subject to the following:
**
** The copyright notices in the Software and this entire statement, including
** the above license grant, this restriction and the following disclaimer,
** must be included in all copies of the Software, in whole or in part, and
** all derivative works of the Software, unless such copies or derivative
** works are solely in the form of machine-executable object code generated by
** a source language processor.
**
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
** DEALINGS IN THE SOFTWARE.
** -LICENSE-END-
*/

#include "DeckLinkAPI.h"
#include "Config.h"

class TestPattern : public IDeckLinkVideoOutputCallback, public IDeckLinkAudioOutputCallback
{
private:
	int32_t					m_refCount;
	BMDConfig*				m_config;
	bool					m_running;
	IDeckLink*				m_deckLink;
	IDeckLinkOutput*		m_deckLinkOutput;
	IDeckLinkDisplayMode*	m_displayMode;

	unsigned long			m_frameWidth;
	unsigned long			m_frameHeight;
	BMDTimeValue			m_frameDuration;
	BMDTimeScale			m_frameTimescale;
	unsigned long			m_framesPerSecond;
	IDeckLinkMutableVideoFrame*	m_videoFrameBlack;
	IDeckLinkMutableVideoFrame*	m_videoFrameWhite;
	IDeckLinkMutableVideoFrame*	m_videoFrameBars;
	unsigned long			m_totalFramesScheduled;
	unsigned long			m_totalFramesDropped;
	unsigned long			m_totalFramesCompleted;

	void*					m_audioBuffer;
	unsigned long			m_audioBufferSampleLength;
	unsigned long			m_audioBufferOffset;
	BMDAudioSampleRate		m_audioSampleRate;

	~TestPattern();

	// Signal Generator Implementation
	void			StartRunning();
	void			StopRunning();
	void			ScheduleNextFrame(bool prerolling);
	void			WriteNextAudioSamples();

	void			PrintStatusLine();

	/* LTN */
	pthread_mutex_t	m_msg_mutex;
	int m_msg_line;
	int m_msg_data_length; /* In Bytes */
	unsigned char m_msg_data[1024];
	/* End: LTN */

public:
	TestPattern(BMDConfig *config);
	bool Run();

	/* LTN */
	char *m_displayModeName;
	/* End: LTN */

	unsigned long getScheduledFrameCount() { return m_totalFramesCompleted; };

	void sendVANCMessage(int linenr, unsigned char *data, int lengthBytes)
	{
		/* Need a mutex here, else racey */
		pthread_mutex_lock(&m_msg_mutex);
		m_msg_line = linenr;
		memcpy(&m_msg_data[0], data, lengthBytes);
		m_msg_data_length = lengthBytes;
		pthread_mutex_unlock(&m_msg_mutex);
	};

	// *** DeckLink API implementation of IDeckLinkVideoOutputCallback IDeckLinkAudioOutputCallback *** //
	// IUnknown
	virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID iid, LPVOID *ppv);
	virtual ULONG STDMETHODCALLTYPE AddRef();
	virtual ULONG STDMETHODCALLTYPE Release();

	virtual HRESULT STDMETHODCALLTYPE ScheduledFrameCompleted(IDeckLinkVideoFrame* completedFrame, BMDOutputFrameCompletionResult result);
	virtual HRESULT STDMETHODCALLTYPE ScheduledPlaybackHasStopped();

	virtual HRESULT STDMETHODCALLTYPE RenderAudioSamples(bool preroll);

	HRESULT CreateFrame(IDeckLinkMutableVideoFrame** theFrame, void (*fillFunc)(IDeckLinkMutableVideoFrame*));
};

void FillSine(void* audioBuffer, unsigned long samplesToWrite, unsigned long channels, unsigned long sampleDepth);
void FillColourBars(IDeckLinkMutableVideoFrame* theFrame, bool reverse);
static inline void FillForwardColourBars(IDeckLinkMutableVideoFrame* theFrame)
{
	FillColourBars(theFrame, false);
}
static inline void FillReverseColourBars(IDeckLinkMutableVideoFrame* theFrame)
{
	FillColourBars(theFrame, true);
}
void FillBlack(IDeckLinkMutableVideoFrame* theFrame);
void FillWhite(IDeckLinkMutableVideoFrame* theFrame);
int GetBytesPerPixel(BMDPixelFormat pixelFormat);
